@@ -1,17 +1,16 @@
-use chrono::{DateTime, Utc};
 use std::cell::RefCell;
 use std::collections::HashSet;
-use std::convert::TryInto;
 use std::fmt::{self, Display, Formatter};
-use std::io::{Read, Seek};
+use std::io::{Cursor, Read, Seek};
 use std::result::Result;
 
 use crate::codetables::{
-    CodeTable3_1, CodeTable4_0, CodeTable4_1, CodeTable4_2, CodeTable4_3, CodeTable4_4,
-    CodeTable5_0, Lookup,
+    CodeTable3_1, CodeTable4_0, CodeTable4_1, CodeTable4_2, CodeTable4_3, CodeTable5_0, Lookup,
 };
-use crate::decoder::{self, DecodeError};
-use crate::reader::{Grib2Read, ParseError, SeekableGrib2Reader};
+use crate::datatypes::*;
+use crate::decoders;
+use crate::error::*;
+use crate::reader::{Grib2Read, SeekableGrib2Reader};
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct SectionInfo {
@@ -51,171 +50,14 @@ impl SectionBody {
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub struct Indicator {
-    /// Discipline - GRIB Master Table Number (see Code Table 0.0)
-    pub discipline: u8,
-    /// Total length of GRIB message in octets (including Section 0)
-    pub total_length: u64,
-}
-
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub struct Identification {
-    /// Identification of originating/generating centre (see Common Code Table C-1)
-    pub centre_id: u16,
-    /// Identification of originating/generating sub-centre (allocated by originating/ generating centre)
-    pub subcentre_id: u16,
-    /// GRIB Master Tables Version Number (see Code Table 1.0)
-    pub master_table_version: u8,
-    /// GRIB Local Tables Version Number (see Code Table 1.1)
-    pub local_table_version: u8,
-    /// Significance of Reference Time (see Code Table 1.2)
-    pub ref_time_significance: u8,
-    /// Reference time of data
-    pub ref_time: DateTime<Utc>,
-    /// Production status of processed data in this GRIB message
-    /// (see Code Table 1.3)
-    pub prod_status: u8,
-    /// Type of processed data in this GRIB message (see Code Table 1.4)
-    pub data_type: u8,
-}
-
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub struct GridDefinition {
-    /// Number of data points
-    pub num_points: u32,
-    /// Grid Definition Template Number
-    pub grid_tmpl_num: u16,
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub struct ProdDefinition {
-    /// Number of coordinate values after Template
-    pub num_coordinates: u16,
-    /// Product Definition Template Number
-    pub prod_tmpl_num: u16,
-    pub(crate) templated: Box<[u8]>,
-    pub(crate) template_supported: bool,
-}
-
-impl ProdDefinition {
-    /// Use [CodeTable4_1] to get textual representation of the returned numerical value.
-    pub fn parameter_category(&self) -> Option<u8> {
-        if self.template_supported {
-            self.templated.get(0).map(|v| *v)
-        } else {
-            None
-        }
-    }
-
-    /// Use [CodeTable4_2] to get textual representation of the returned numerical value.
-    pub fn parameter_number(&self) -> Option<u8> {
-        if self.template_supported {
-            self.templated.get(1).map(|v| *v)
-        } else {
-            None
-        }
-    }
-
-    /// Use [CodeTable4_3] to get textual representation of the returned numerical value.
-    pub fn generating_process(&self) -> Option<u8> {
-        if self.template_supported {
-            let index = match self.prod_tmpl_num {
-                0..=39 => Some(2),
-                40..=43 => Some(4),
-                44..=46 => Some(15),
-                47 => Some(2),
-                48..=49 => Some(26),
-                51 => Some(2),
-                // 53 and 54 is variable and not supported as of now
-                55..=56 => Some(8),
-                // 57 and 58 is variable and not supported as of now
-                59 => Some(8),
-                60..=61 => Some(2),
-                62..=63 => Some(8),
-                // 67 and 68 is variable and not supported as of now
-                70..=73 => Some(7),
-                76..=79 => Some(5),
-                80..=81 => Some(27),
-                82 => Some(16),
-                83 => Some(2),
-                84 => Some(16),
-                85 => Some(15),
-                86..=91 => Some(2),
-                254 => Some(2),
-                1000..=1101 => Some(2),
-                _ => None,
-            }?;
-            self.templated.get(index).map(|v| *v)
-        } else {
-            None
-        }
-    }
-
-    /// Returns the unit and value of the forecast time wrapped by `Option`.
-    /// Use [CodeTable4_4] to get textual representation of the unit.
-    pub fn forecast_time(&self) -> Option<(u8, u32)> {
-        if self.template_supported {
-            let unit_index = match self.prod_tmpl_num {
-                0..=15 => Some(8),
-                32..=34 => Some(8),
-                40..=43 => Some(10),
-                44..=47 => Some(21),
-                48..=49 => Some(32),
-                51 => Some(8),
-                // 53 and 54 is variable and not supported as of now
-                55..=56 => Some(14),
-                // 57 and 58 is variable and not supported as of now
-                59 => Some(14),
-                60..=61 => Some(8),
-                62..=63 => Some(14),
-                // 67 and 68 is variable and not supported as of now
-                70..=73 => Some(13),
-                76..=79 => Some(11),
-                80..=81 => Some(33),
-                82..=84 => Some(22),
-                85 => Some(21),
-                86..=87 => Some(8),
-                88 => Some(26),
-                91 => Some(8),
-                1000..=1101 => Some(8),
-                _ => None,
-            }?;
-            let unit = self.templated.get(unit_index).map(|v| *v);
-            let start = unit_index + 1;
-            let end = unit_index + 5;
-            let time = u32::from_be_bytes(self.templated[start..end].try_into().unwrap());
-            unit.zip(Some(time))
-        } else {
-            None
-        }
-    }
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub struct ReprDefinition {
-    /// Number of data points where one or more values are
-    /// specified in Section 7 when a bit map is present, total
-    /// number of data points when a bit map is absent
-    pub num_points: u32,
-    /// Data Representation Template Number
-    pub repr_tmpl_num: u16,
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub struct BitMap {
-    /// Bit-map indicator
-    pub bitmap_indicator: u8,
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-struct SubMessageIndex {
-    section2: Option<usize>,
-    section3: usize,
-    section4: usize,
-    section5: usize,
-    section6: usize,
-    section7: usize,
+pub(crate) struct SubMessageIndex {
+    pub(crate) section2: Option<usize>,
+    pub(crate) section3: usize,
+    pub(crate) section4: usize,
+    pub(crate) section5: usize,
+    pub(crate) section6: usize,
+    pub(crate) section7: usize,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
@@ -238,20 +80,31 @@ impl Display for TemplateInfo {
     }
 }
 
+pub fn from_reader<SR: Read + Seek>(
+    reader: SR,
+) -> Result<Grib2<SeekableGrib2Reader<SR>>, GribError> {
+    Grib2::<SeekableGrib2Reader<SR>>::read_with_seekable(reader)
+}
+
+pub fn from_slice(bytes: &[u8]) -> Result<Grib2<SeekableGrib2Reader<Cursor<&[u8]>>>, GribError> {
+    let reader = Cursor::new(bytes);
+    Grib2::<SeekableGrib2Reader<Cursor<&[u8]>>>::read_with_seekable(reader)
+}
+
 pub struct Grib2<R> {
-    reader: RefCell<R>,
-    sections: Box<[SectionInfo]>,
-    submessages: Box<[SubMessageIndex]>,
+    pub(crate) reader: RefCell<R>,
+    pub(crate) sections: Box<[SectionInfo]>,
+    pub(crate) submessages: Box<[SubMessageIndex]>,
 }
 
 impl<R: Grib2Read> Grib2<R> {
     pub fn read(mut r: R) -> Result<Self, GribError> {
-        let sects = r.scan()?;
-        let submessages = index_submessages(&sects)?;
+        let sections = r.scan()?;
+        let submessages = index_submessages(&sections)?;
         Ok(Self {
             reader: RefCell::new(r),
-            sections: sects,
-            submessages: submessages,
+            sections,
+            submessages,
         })
     }
 
@@ -278,6 +131,12 @@ impl<R: Grib2Read> Grib2<R> {
         }
     }
 
+    /// Iterates over submessages.
+    #[inline]
+    pub fn iter(&self) -> SubMessageIterator {
+        self.submessages()
+    }
+
     pub fn submessages(&self) -> SubMessageIterator {
         SubMessageIterator::new(&self.submessages, &self.sections)
     }
@@ -297,11 +156,11 @@ impl<R: Grib2Read> Grib2<R> {
             .ok_or(GribError::InternalDataError)?;
 
         let reader = self.reader.borrow_mut();
-        let values = decoder::dispatch(sect5, sect6, sect7, reader)?;
+        let values = decoders::dispatch(sect5, sect6, sect7, reader)?;
         Ok(values)
     }
 
-    pub fn sections(&self) -> &Box<[SectionInfo]> {
+    pub fn sections(&self) -> &[SectionInfo] {
         &self.sections
     }
 
@@ -312,9 +171,7 @@ impl<R: Grib2Read> Grib2<R> {
 
 /// Validates the section order of sections and split them into a
 /// vector of section groups.
-fn index_submessages(
-    sects: &Box<[SectionInfo]>,
-) -> Result<Box<[SubMessageIndex]>, ValidationError> {
+fn index_submessages(sects: &[SectionInfo]) -> Result<Box<[SubMessageIndex]>, ValidationError> {
     let mut iter = sects.iter().enumerate();
     let mut starts = Vec::new();
     let mut i2_default = None;
@@ -415,7 +272,7 @@ fn index_submessages(
     Ok(starts.into_boxed_slice())
 }
 
-fn get_templates(sects: &Box<[SectionInfo]>) -> Vec<TemplateInfo> {
+fn get_templates(sects: &[SectionInfo]) -> Vec<TemplateInfo> {
     let uniq: HashSet<_> = sects.iter().filter_map(|s| s.get_tmpl_code()).collect();
     let mut vec: Vec<_> = uniq.into_iter().collect();
     vec.sort_unstable();
@@ -424,16 +281,16 @@ fn get_templates(sects: &Box<[SectionInfo]>) -> Vec<TemplateInfo> {
 
 #[derive(Clone)]
 pub struct SubMessageIterator<'a> {
-    indices: &'a Box<[SubMessageIndex]>,
-    sections: &'a Box<[SectionInfo]>,
+    indices: &'a [SubMessageIndex],
+    sections: &'a [SectionInfo],
     pos: usize,
 }
 
 impl<'a> SubMessageIterator<'a> {
-    fn new(indices: &'a Box<[SubMessageIndex]>, sections: &'a Box<[SectionInfo]>) -> Self {
+    fn new(indices: &'a [SubMessageIndex], sections: &'a [SectionInfo]) -> Self {
         Self {
-            indices: indices,
-            sections: sections,
+            indices,
+            sections,
             pos: 0,
         }
     }
@@ -507,7 +364,25 @@ impl<'a> SubMessage<'a> {
 
     pub fn describe(&self) -> String {
         let category = self.prod_def().parameter_category();
-        let forecast_time = self.prod_def().forecast_time();
+        let forecast_time = self
+            .prod_def()
+            .forecast_time()
+            .map(|ft| ft.describe())
+            .unwrap_or((String::new(), String::new()));
+        let fixed_surfaces_info = self
+            .prod_def()
+            .fixed_surfaces()
+            .map(|(first, second)| (first.describe(), second.describe()))
+            .map(|(first, second)| (first.0, first.1, first.2, second.0, second.1, second.2))
+            .unwrap_or((
+                String::new(),
+                String::new(),
+                String::new(),
+                String::new(),
+                String::new(),
+                String::new(),
+            ));
+
         format!(
             "\
 Grid:                                   {}
@@ -517,33 +392,41 @@ Product:                                {}
   Generating Proceess:                  {}
   Forecast Time:                        {}
   Forecast Time Unit:                   {}
+  1st Fixed Surface Type:               {}
+  1st Scale Factor:                     {}
+  1st Scaled Value:                     {}
+  2nd Fixed Surface Type:               {}
+  2nd Scale Factor:                     {}
+  2nd Scaled Value:                     {}
 Data Representation:                    {}
 ",
-            self.3.describe().unwrap_or(String::new()),
-            self.4.describe().unwrap_or(String::new()),
+            self.3.describe().unwrap_or_default(),
+            self.4.describe().unwrap_or_default(),
             category
                 .map(|v| CodeTable4_1::new(self.indicator().discipline)
                     .lookup(usize::from(v))
                     .to_string())
-                .unwrap_or(String::new()),
+                .unwrap_or_default(),
             self.prod_def()
                 .parameter_number()
                 .zip(category)
                 .map(|(n, c)| CodeTable4_2::new(self.indicator().discipline, c)
                     .lookup(usize::from(n))
                     .to_string())
-                .unwrap_or(String::new()),
+                .unwrap_or_default(),
             self.prod_def()
                 .generating_process()
                 .map(|v| CodeTable4_3.lookup(usize::from(v)).to_string())
-                .unwrap_or(String::new()),
-            forecast_time
-                .map(|(_, v)| v.to_string())
-                .unwrap_or(String::new()),
-            forecast_time
-                .map(|(unit, _)| CodeTable4_4.lookup(usize::from(unit)).to_string())
-                .unwrap_or(String::new()),
-            self.5.describe().unwrap_or(String::new()),
+                .unwrap_or_default(),
+            forecast_time.1,
+            forecast_time.0,
+            fixed_surfaces_info.0,
+            fixed_surfaces_info.1,
+            fixed_surfaces_info.2,
+            fixed_surfaces_info.3,
+            fixed_surfaces_info.4,
+            fixed_surfaces_info.5,
+            self.5.describe().unwrap_or_default(),
         )
     }
 }
@@ -555,10 +438,7 @@ pub struct SubMessageSection<'a> {
 
 impl<'a> SubMessageSection<'a> {
     pub fn new(index: usize, body: &'a SectionInfo) -> Self {
-        Self {
-            index: index,
-            body: body,
-        }
+        Self { index, body }
     }
 
     pub fn template_code(&self) -> Option<TemplateInfo> {
@@ -570,64 +450,13 @@ impl<'a> SubMessageSection<'a> {
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub enum GribError {
-    InternalDataError,
-    ParseError(ParseError),
-    ValidationError(ValidationError),
-    DecodeError(DecodeError),
-}
-
-impl From<ParseError> for GribError {
-    fn from(e: ParseError) -> Self {
-        Self::ParseError(e)
-    }
-}
-
-impl From<ValidationError> for GribError {
-    fn from(e: ValidationError) -> Self {
-        Self::ValidationError(e)
-    }
-}
-
-impl From<DecodeError> for GribError {
-    fn from(e: DecodeError) -> Self {
-        Self::DecodeError(e)
-    }
-}
-
-impl Display for GribError {
-    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
-        match self {
-            Self::InternalDataError => write!(f, "Something unexpected happend"),
-            Self::ParseError(e) => write!(f, "{}", e),
-            Self::ValidationError(e) => write!(f, "{}", e),
-            Self::DecodeError(e) => write!(f, "{:#?}", e),
-        }
-    }
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub enum ValidationError {
-    GRIB2IterationSuddenlyFinished,
-    NoGridDefinition(usize),
-    GRIB2WrongIteration(usize),
-}
-
-impl Display for ValidationError {
-    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
-        match self {
-            Self::GRIB2IterationSuddenlyFinished => write!(f, "GRIB2 file suddenly finished"),
-            Self::NoGridDefinition(i) => write!(f, "Grid Definition Section not found at {}", i),
-            Self::GRIB2WrongIteration(i) => write!(f, "GRIB2 sections wrongly ordered at {}", i),
-        }
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
 
+    use std::fs::File;
+    use std::io::BufReader;
+
     macro_rules! sect_placeholder {
         ($num:expr) => {{
             SectionInfo {
@@ -649,6 +478,30 @@ mod tests {
         }}
     }
 
+    #[test]
+    fn from_buf_reader() {
+        let f = File::open(
+            "testdata/icon_global_icosahedral_single-level_2021112018_000_TOT_PREC.grib2",
+        )
+        .unwrap();
+        let f = BufReader::new(f);
+        let result = from_reader(f);
+        assert!(result.is_ok())
+    }
+
+    #[test]
+    fn from_bytes() {
+        let f = File::open(
+            "testdata/icon_global_icosahedral_single-level_2021112018_000_TOT_PREC.grib2",
+        )
+        .unwrap();
+        let mut f = BufReader::new(f);
+        let mut buf = Vec::new();
+        f.read_to_end(&mut buf).unwrap();
+        let result = from_slice(&buf);
+        assert!(result.is_ok())
+    }
+
     #[test]
     fn index_submessages_simple() {
         let sects = sect_list![0, 1, 2, 3, 4, 5, 6, 7, 8,];
@@ -1023,22 +876,4 @@ mod tests {
             ]
         );
     }
-
-    #[test]
-    fn prod_definition_parameters() {
-        let data = ProdDefinition {
-            num_coordinates: 0,
-            prod_tmpl_num: 0,
-            templated: vec![
-                193, 0, 2, 153, 255, 0, 0, 0, 0, 0, 0, 0, 40, 1, 255, 255, 255, 255, 255, 255, 255,
-                255, 255, 255, 255,
-            ]
-            .into_boxed_slice(),
-            template_supported: true,
-        };
-
-        assert_eq!(data.parameter_category(), Some(193));
-        assert_eq!(data.parameter_number(), Some(0));
-        assert_eq!(data.forecast_time(), Some((0, 40)));
-    }
 }
