@@ -5,7 +5,7 @@ use tempfile::TempDir;
 
 mod utils;
 
-const CMD_NAME: &'static str = "gribber";
+const CMD_NAME: &str = "gribber";
 
 #[test]
 fn help() -> Result<(), Box<dyn std::error::Error>> {
@@ -15,7 +15,7 @@ fn help() -> Result<(), Box<dyn std::error::Error>> {
         .success()
         .stdout(
             predicate::str::contains("USAGE:")
-                .and(predicate::str::contains("FLAGS:"))
+                .and(predicate::str::contains("OPTIONS:"))
                 .and(predicate::str::contains("SUBCOMMANDS:")),
         )
         .stderr(predicate::str::is_empty());
@@ -28,13 +28,13 @@ fn no_subcommand_specified() -> Result<(), Box<dyn std::error::Error>> {
     let mut cmd = Command::cargo_bin(CMD_NAME)?;
     cmd.arg("--help");
     let help_msg = cmd.output()?.stdout;
-    let help_msg = format!("{}", String::from_utf8(help_msg)?);
+    let help_msg = String::from_utf8(help_msg)?;
 
     let mut cmd = Command::cargo_bin(CMD_NAME)?;
     cmd.assert()
         .failure()
         .stdout(predicate::str::is_empty())
-        .stderr(predicate::str::similar(help_msg));
+        .stderr(predicate::str::diff(help_msg));
 
     Ok(())
 }
@@ -79,7 +79,7 @@ Type of processed data:                 Analysis and forecast products
     cmd.arg("info").arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(out_str))
+        .stdout(predicate::str::diff(out_str))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -90,21 +90,21 @@ fn list() -> Result<(), Box<dyn std::error::Error>> {
     let tempfile = utils::jma_tornado_nowcast_file()?;
     let arg_path = tempfile.path();
 
-    let out_str = "   id │ Parameter                       Generating process  Forecast time
-    0 │ code '0' is not implemented     Analysis                 0 Minute
-    1 │ code '0' is not implemented     Forecast                10 Minute
-    2 │ code '0' is not implemented     Forecast                20 Minute
-    3 │ code '0' is not implemented     Forecast                30 Minute
-    4 │ code '0' is not implemented     Forecast                40 Minute
-    5 │ code '0' is not implemented     Forecast                50 Minute
-    6 │ code '0' is not implemented     Forecast                60 Minute
+    let out_str = "   id │ Parameter                       Generating process  Forecast time 1st fixed surface 2nd fixed surface
+    0 │ code '0' is not implemented     Analysis                    0 [m]               NaN               NaN
+    1 │ code '0' is not implemented     Forecast                   10 [m]               NaN               NaN
+    2 │ code '0' is not implemented     Forecast                   20 [m]               NaN               NaN
+    3 │ code '0' is not implemented     Forecast                   30 [m]               NaN               NaN
+    4 │ code '0' is not implemented     Forecast                   40 [m]               NaN               NaN
+    5 │ code '0' is not implemented     Forecast                   50 [m]               NaN               NaN
+    6 │ code '0' is not implemented     Forecast                   60 [m]               NaN               NaN
 ";
 
     let mut cmd = Command::cargo_bin(CMD_NAME)?;
     cmd.arg("list").arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(out_str))
+        .stdout(predicate::str::diff(out_str))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -124,6 +124,12 @@ Product:                                Analysis or forecast at a horizontal lev
   Generating Proceess:                  Analysis
   Forecast Time:                        0
   Forecast Time Unit:                   Minute
+  1st Fixed Surface Type:               Ground or water surface
+  1st Scale Factor:                     Missing
+  1st Scaled Value:                     Missing
+  2nd Fixed Surface Type:               code '255' is not implemented
+  2nd Scale Factor:                     Missing
+  2nd Scaled Value:                     Missing
 Data Representation:                    Run length packing with level values
 
 1
@@ -134,6 +140,12 @@ Product:                                Analysis or forecast at a horizontal lev
   Generating Proceess:                  Forecast
   Forecast Time:                        10
   Forecast Time Unit:                   Minute
+  1st Fixed Surface Type:               Ground or water surface
+  1st Scale Factor:                     Missing
+  1st Scaled Value:                     Missing
+  2nd Fixed Surface Type:               code '255' is not implemented
+  2nd Scale Factor:                     Missing
+  2nd Scaled Value:                     Missing
 Data Representation:                    Run length packing with level values
 
 2
@@ -144,6 +156,12 @@ Product:                                Analysis or forecast at a horizontal lev
   Generating Proceess:                  Forecast
   Forecast Time:                        20
   Forecast Time Unit:                   Minute
+  1st Fixed Surface Type:               Ground or water surface
+  1st Scale Factor:                     Missing
+  1st Scaled Value:                     Missing
+  2nd Fixed Surface Type:               code '255' is not implemented
+  2nd Scale Factor:                     Missing
+  2nd Scaled Value:                     Missing
 Data Representation:                    Run length packing with level values
 
 3
@@ -154,6 +172,12 @@ Product:                                Analysis or forecast at a horizontal lev
   Generating Proceess:                  Forecast
   Forecast Time:                        30
   Forecast Time Unit:                   Minute
+  1st Fixed Surface Type:               Ground or water surface
+  1st Scale Factor:                     Missing
+  1st Scaled Value:                     Missing
+  2nd Fixed Surface Type:               code '255' is not implemented
+  2nd Scale Factor:                     Missing
+  2nd Scaled Value:                     Missing
 Data Representation:                    Run length packing with level values
 
 4
@@ -164,6 +188,12 @@ Product:                                Analysis or forecast at a horizontal lev
   Generating Proceess:                  Forecast
   Forecast Time:                        40
   Forecast Time Unit:                   Minute
+  1st Fixed Surface Type:               Ground or water surface
+  1st Scale Factor:                     Missing
+  1st Scaled Value:                     Missing
+  2nd Fixed Surface Type:               code '255' is not implemented
+  2nd Scale Factor:                     Missing
+  2nd Scaled Value:                     Missing
 Data Representation:                    Run length packing with level values
 
 5
@@ -174,6 +204,12 @@ Product:                                Analysis or forecast at a horizontal lev
   Generating Proceess:                  Forecast
   Forecast Time:                        50
   Forecast Time Unit:                   Minute
+  1st Fixed Surface Type:               Ground or water surface
+  1st Scale Factor:                     Missing
+  1st Scaled Value:                     Missing
+  2nd Fixed Surface Type:               code '255' is not implemented
+  2nd Scale Factor:                     Missing
+  2nd Scaled Value:                     Missing
 Data Representation:                    Run length packing with level values
 
 6
@@ -184,6 +220,12 @@ Product:                                Analysis or forecast at a horizontal lev
   Generating Proceess:                  Forecast
   Forecast Time:                        60
   Forecast Time Unit:                   Minute
+  1st Fixed Surface Type:               Ground or water surface
+  1st Scale Factor:                     Missing
+  1st Scaled Value:                     Missing
+  2nd Fixed Surface Type:               code '255' is not implemented
+  2nd Scale Factor:                     Missing
+  2nd Scaled Value:                     Missing
 Data Representation:                    Run length packing with level values
 
 ";
@@ -192,7 +234,7 @@ Data Representation:                    Run length packing with level values
     cmd.arg("list").arg("-d").arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(out_str))
+        .stdout(predicate::str::diff(out_str))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -239,7 +281,7 @@ Sections:
    31 │ 000000000000284d - 0000000000002851 │ Section 8
 
 SubMessages:
-   id │    S2    S3    S4    S5    S6    S7 │ Tmpl3   Tmpl4   Tmpl5
+   id │    S2    S3    S4    S5    S6    S7 │ Tmpl3   Tmpl4   Tmpl5  
     0 │     -     2     3     4     5     6 │ 3.0     4.0     5.200  
     1 │     -     2     7     8     9    10 │ 3.0     4.0     5.200  
     2 │     -     2    11    12    13    14 │ 3.0     4.0     5.200  
@@ -258,7 +300,7 @@ Templates:
     cmd.arg("inspect").arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(out_str))
+        .stdout(predicate::str::diff(out_str))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -307,7 +349,7 @@ fn inspect_with_opt_s() -> Result<(), Box<dyn std::error::Error>> {
     cmd.arg("inspect").arg("-s").arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(out_str))
+        .stdout(predicate::str::diff(out_str))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -318,7 +360,7 @@ fn inspect_with_opt_m() -> Result<(), Box<dyn std::error::Error>> {
     let tempfile = utils::jma_tornado_nowcast_file()?;
     let arg_path = tempfile.path();
 
-    let out_str = "   id │    S2    S3    S4    S5    S6    S7 │ Tmpl3   Tmpl4   Tmpl5
+    let out_str = "   id │    S2    S3    S4    S5    S6    S7 │ Tmpl3   Tmpl4   Tmpl5  
     0 │     -     2     3     4     5     6 │ 3.0     4.0     5.200  
     1 │     -     2     7     8     9    10 │ 3.0     4.0     5.200  
     2 │     -     2    11    12    13    14 │ 3.0     4.0     5.200  
@@ -332,7 +374,7 @@ fn inspect_with_opt_m() -> Result<(), Box<dyn std::error::Error>> {
     cmd.arg("inspect").arg("-m").arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(out_str))
+        .stdout(predicate::str::diff(out_str))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -353,7 +395,7 @@ fn inspect_with_opt_t() -> Result<(), Box<dyn std::error::Error>> {
     cmd.arg("inspect").arg("-t").arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(out_str))
+        .stdout(predicate::str::diff(out_str))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -367,7 +409,7 @@ fn inspect_with_all_opts() -> Result<(), Box<dyn std::error::Error>> {
     let mut cmd = Command::cargo_bin(CMD_NAME)?;
     cmd.arg("inspect").arg(arg_path);
     let msg_no_opt = cmd.output()?.stdout;
-    let msg_no_opt = format!("{}", String::from_utf8(msg_no_opt)?);
+    let msg_no_opt = String::from_utf8(msg_no_opt)?;
 
     let mut cmd = Command::cargo_bin(CMD_NAME)?;
     cmd.arg("inspect")
@@ -377,7 +419,7 @@ fn inspect_with_all_opts() -> Result<(), Box<dyn std::error::Error>> {
         .arg(arg_path);
     cmd.assert()
         .success()
-        .stdout(predicate::str::similar(msg_no_opt))
+        .stdout(predicate::str::diff(msg_no_opt))
         .stderr(predicate::str::is_empty());
 
     Ok(())
@@ -572,6 +614,39 @@ fn decode_complex_packing_as_little_endian() -> Result<(), Box<dyn std::error::E
     Ok(())
 }
 
+#[test]
+fn decode_jpeg2000_code_stream_as_little_endian() -> Result<(), Box<dyn std::error::Error>> {
+    let arg_path = utils::cmc_glb_file_path();
+
+    let dir = TempDir::new()?;
+    let out_path = dir.path().join("out.bin");
+    let out_path = format!("{}", out_path.display());
+
+    let mut cmd = Command::cargo_bin(CMD_NAME)?;
+    cmd.arg("decode")
+        .arg(arg_path)
+        .arg("0")
+        .arg("-l")
+        .arg(&out_path);
+    cmd.assert()
+        .success()
+        .stdout(predicate::str::is_empty())
+        .stderr(predicate::str::is_empty());
+
+    // Compares integer values encoded using simple packing since there are some differences
+    // between float values from gribber and wgrib2.
+    let ref_val = f32::from_be_bytes([0x45, 0x0e, 0xcc, 0x05]);
+    let exp: i16 = -2;
+    let dig: i16 = 1;
+    let expected = utils::cmc_glb_le_bin_bytes()?;
+    let expected = utils::encode_le_bytes_using_simple_packing(expected, ref_val, exp, dig);
+    let actual = utils::cat_as_bytes(&out_path)?;
+    let actual = utils::encode_le_bytes_using_simple_packing(actual, ref_val, exp, dig);
+    assert_eq!(actual, expected);
+
+    Ok(())
+}
+
 macro_rules! test_subcommands_without_args {
     ($(($name:ident, $str:expr),)*) => ($(
         #[test]
@@ -637,7 +712,7 @@ macro_rules! test_subcommands_with_non_grib {
             cmd.assert()
                 .failure()
                 .stdout(predicate::str::is_empty())
-                .stderr(predicate::str::similar("Not GRIB data\n"));
+                .stderr(predicate::str::diff("Not GRIB data\n"));
 
             Ok(())
         }
@@ -662,7 +737,7 @@ macro_rules! test_subcommands_with_too_small_file {
             cmd.assert()
                 .failure()
                 .stdout(predicate::str::is_empty())
-                .stderr(predicate::str::similar(
+                .stderr(predicate::str::diff(
                     "Error in checking file type: failed to fill whole buffer\n",
                 ));
 
